﻿Public Class frmMain
    ' Define the K8000 object - allow this object to raise events
    Private WithEvents K8000 As New Remote_K8000()

    ' Class variabled
    Private _isConnected As Boolean
    Private toggleOn As Boolean = False

    Private result As Boolean
    Private chkBoxes() As CheckBox = New CheckBox() {IO1, IO2, IO3, IO4, IO5, IO6, IO7, IO8, IO9, IO10, IO11, IO12, IO13, IO14, IO15, IO16}

    ' this variable indicates if the click events for the checkboxes should be supressed
    Private supressCheckboxClickEvents As Boolean = False

    ' Form button values 
    Private Const BTN_CONNECT = "Connect"
    Private Const BTN_DISCONNECT = "Disconnect"


    ''' <summary>
    ''' This property will tells if the K8000 is currently connected
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property isConnected As Boolean
        Get
            Return _isConnected
        End Get
        Set(ByVal value As Boolean)
            value = _isConnected
        End Set
    End Property



#Region "Connection Code"
    Dim host As String = ""
    Dim password As String = ""
    Dim port As Integer

    ''' <summary>
    ''' This button will create a connection with the K8000
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CmdConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdConnect.Click
        If BGW.IsBusy = False Then
            If K8000.isConnected Then
                ' disconnect from the server
                K8000.disconnect()
                cmdConnect.Text = BTN_CONNECT
                MsgBox("K8000 card disconnected", MsgBoxStyle.OkOnly + MsgBoxStyle.Information, "K8000 disconnected")
            Else
                ' make a connection but validate the inputs first
                If validateInputs(host, port, password) Then
                    BGW.RunWorkerAsync()
                    Me.Cursor = Cursors.WaitCursor
                End If
            End If
        Else
            MsgBox("There is already a connection attempt in progress! Please retry in a few moments...", MsgBoxStyle.OkOnly + MsgBoxStyle.Information, "OK")
        End If
    End Sub


    ''' <summary>
    ''' validateInputs will validate the input fields. The results will be passed by the byref values if the
    ''' function call succeeded.
    ''' </summary>
    ''' <param name="host">the hostname</param>
    ''' <param name="port">the portname</param>
    ''' <param name="password">the password</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function validateInputs(ByRef host As String, ByRef port As Integer, ByRef password As String) As Boolean
        Dim valid As Boolean = True
        EP.Clear()

        Dim portname As Integer
        If Integer.TryParse(inpPort.Text, portname) Then
            If portname > 0 And portname < 65536 Then
                port = portname
            Else
                EP.SetError(inpPort, "Type a TCP/IP server port between 1 and 65535!")
                valid = False
            End If
        Else
            EP.SetError(inpPort, "Type a TCP/IP server port between 1 and 65535!")
            valid = False
        End If

        If inpHost.Text().Trim() = "" Then
            valid = False
            EP.SetError(inpHost, "Set a valid hostname or IP address")
        Else
            host = inpHost.Text.Trim()
        End If

        If inpPassword.Text.Trim = "" Or InStr(inpPassword.Text().Trim(), " ") > 0 Then
            valid = False
            EP.SetError(inpPassword, "Please type a valid password. A password may not contain spaces!")
        Else
            password = inpPassword.Text.Trim()
        End If

        Return valid
    End Function




    ''' <summary>
    ''' BGW_DoWork contains the code that must be executed in the background thread
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub BGW_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BGW.DoWork
        result = K8000.connect(host, port, password)
    End Sub



    ''' <summary>
    ''' BGW_RunWorkerCompleted is called when the background thread processed all the data
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub BGW_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BGW.RunWorkerCompleted
        ' Reset the cursor
        Me.Cursor = Cursors.Default

        If result Then
            cmdConnect.Text = BTN_DISCONNECT
            isConnected = True
            lblStatus.Text = "Connection established"
        Else
            cmdConnect.Text = BTN_CONNECT
            MsgBox("The connection with the server could not be established! " & vbCrLf & "Reason: " & _
                   K8000.lastException.Message(), _
                    MsgBoxStyle.OkOnly + MsgBoxStyle.Exclamation, _
                    "Connection failed")
            isConnected = False
            lblStatus.Text = "Connection failed"
        End If
    End Sub
#End Region





#Region "GUI to K8000 commands"
    ''' <summary>
    ''' The setAll_Click is executed when the user clicks the Set All button
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub setAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSetAllIO.Click
        Dim iodata(16) As Boolean

        If K8000.setAllIO(iodata) Then
            reflectIOdata(iodata)
        End If
    End Sub



    ''' <summary>
    ''' The clearAll_Click is executed when the user clicks the Set All button
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub clear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClearAllIO.Click
        Dim iodata(16) As Boolean

        If K8000.clearAllIO(iodata) Then
            reflectIOdata(iodata)
        End If
    End Sub



    ''' <summary>
    ''' IO_CheckedChanged is called when one of the I/O channels is clicked
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub IO_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        If supressCheckboxClickEvents = False Then
            Dim chk As CheckBox = DirectCast(sender, CheckBox)
            Dim selectedChannelStr As String = chk.Name.Substring(2)

            Dim selectedChannel As Integer

            If Integer.TryParse(selectedChannelStr, selectedChannel) Then
                Dim iodata(16) As Boolean

                If chk.Checked Then
                    ' Value is now ON
                    If K8000.setIOchannel(selectedChannel, iodata) Then
                        reflectIOdata(iodata)
                    End If
                Else
                    ' Value is now OFF
                    If K8000.clearIOchannel(selectedChannel, iodata) Then
                        reflectIOdata(iodata)
                    End If
                End If
            Else
                Debug.Print("invalid integer")
            End If
        End If
    End Sub



    ''' <summary>
    ''' DAregulator_MouseUp is called when the mouse is released from the DA regulator control. 
    ''' Then the value is sent over the network. (the value is only sent when the mouse is released
    ''' from the control, to avoid there are too many data transmissions. It would slown down
    ''' the server response.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub DAregulator_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles DAregulator.MouseUp
        Dim selectedValue = DAregulator.Value
        K8000.setDAC(1, selectedValue)
    End Sub



    ''' <summary>
    ''' cmdGetAllIO_Click will update all the I/O data
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdGetAllIO_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetAllIO.Click
        readIOdata()
    End Sub



    ''' <summary>
    ''' readIOdata() is an aider function. It requests all I/O from the K8000 and will make sure
    ''' the program's checkboxes are updated.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub readIOdata()
        Dim iodata(16) As Boolean

        If K8000.getAllIO(iodata) Then
            reflectIOdata(iodata)
        End If
    End Sub



    ''' <summary>
    ''' reflectIOdata() will analyze the IOdata array 
    ''' </summary>
    ''' <param name="IOdata"></param>
    ''' <remarks></remarks>
    Public Sub reflectIOdata(ByVal IOdata() As Boolean)
        If iodata.Length = 17 Then
            ' Only continue if the data has the right length
            supressCheckboxClickEvents = True
            For i As Integer = 1 To iodata.Length - 1
                chkBoxes(i - 1).Checked = iodata(i - 1)
            Next
            supressCheckboxClickEvents = False
        Else
            ' Error. But for now we are not showing it!
            Debug.Print("invalid length of iodata!")
        End If
    End Sub



    ''' <summary>
    ''' CmdAD_Click read the data of all D/A channels from the K8000
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CmdAD_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CmdAD.Click
        K8000.getAllAD(AD1.Value, AD2.Value, AD3.Value, AD4.Value)
    End Sub



    ''' <summary>
    ''' This event occurs when any send command failed.
    ''' The event belongs to the K8000 object! (Notice the withevents keyword).
    ''' I implemented this code to avoid each function has to contain an error handler!
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub K8000_errorInDisconnectedState() Handles K8000.noConnectionError
        MsgBox("Impossible to send command: the K8000 is currently not connected!", _
               MsgBoxStyle.OkOnly + MsgBoxStyle.Information, _
               "Send error")
    End Sub



    ''' <summary>
    ''' This event occurs when the disconnection of the K8000 is detected.
    ''' So far this code doesn't work too good yet.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub K8000_clientDisconnected() Handles K8000.clientDisconnected
        cmdConnect.Text = BTN_CONNECT
        lblStatus.Text = "Client disconnection detected!"
    End Sub
#End Region





#Region "General Form Methods"
    ''' <summary>
    ''' This method is the first method that will be executed when starting the program
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Set initial labels
        lblStatus.Text = "K8000 not connected"

        ' Set A/D progress bars
        AD1.Minimum = 0
        AD1.Maximum = 255
        AD2.Minimum = 0
        AD2.Maximum = 255
        AD3.Minimum = 0
        AD3.Maximum = 255
        AD4.Minimum = 0
        AD4.Maximum = 255

        ' Set the label of the cmd_connect
        cmdConnect.Text = BTN_CONNECT

        ' Assign the events for the checkboxes
        ' Reason: VB.NET doens't support control arrays. This code has the same effect
        ' This array represents an array of checkbox objects. It is the equivalent of the old control array (VB5)
        chkBoxes = New CheckBox() {IO1, IO2, IO3, IO4, IO5, IO6, IO7, IO8, IO9, IO10, IO11, IO12, IO13, IO14, IO15, IO16}

        For Each chk As CheckBox In chkBoxes
            AddHandler chk.CheckedChanged, AddressOf IO_CheckedChanged
        Next

        ' Set the dimensions of the DA regulator
        DAregulator.Minimum = 0
        DAregulator.Maximum = 63

        ' Load the settings
        loadSettings()
    End Sub



    ''' <summary>
    ''' loadSettings() will load all settings from the settings file.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub loadSettings()
        With My.Settings
            inpHost.Text = .host
            inpPort.Text = .port
            inpPassword.Text = .password
        End With
    End Sub



    ''' <summary>
    ''' All application settings are saved when the form is closed
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        If K8000.isConnected Then
            K8000.disconnect()
        End If

        With My.Settings
            .host = inpHost.Text
            .port = inpPort.Text
            .password = inpPassword.Text
            .Save()
        End With
    End Sub
#End Region




End Class
